%%% Matlab code to calculate the magnetic forces from a pair of permanent
%%% magnets, as they are commonly used for single-molecule magnetic tweezers
%%%
%%% This code implements the (semi-)analytical calculations based on the
%%% Biot-Savart law described in
%%% "Quantitative modeling and optimization of magnetic tweezers"
%%% Jan Lipfert, Xiaomin Hao, and Nynke H. Dekker
%%% Biophys J. 2009 Jun 17;96(12):5040-9. doi: 10.1016/j.bpj.2009.03.055.
%%%
%%% For notations, schematic drawings of the geometry, and examples see
%%% the paper by Lipfert et al. Biophys J. (2009), in particular also the
%%% supplementary information.
%%%
%%% The bead parameter for M280 and M270 beads were tested/calibrated in
%%% "Freely orbiting magnetic tweezers to directly monitor changes in the
%%% twist of nucleic acids"
%%% Lipfert J, Wiggin M, Kerssemakers JW, Pedaci F, Dekker NH.
%%% Nat Commun. 2011 Aug 23;2:439. doi: 10.1038/ncomms1450.
%%%
%%% This part of the code computes the forces, assuming that you have
%%% pre-computed the fields!
%%% Please run the companion code "Lipfert_etal_FieldCalculations.m"
%%% first to compute and save the fields for your magnet geometry of
%%% interest.
%%%
%%% If you find this code useful, please cite the references indicated above.
%%%
%%% Author: Jan Lipfert


clf;clc;clear;
hold on; box on;

%%% Select the bead type:
%%% 1 = MyOne beads (~ 1 um diameter)
%%% 2 = M280 beads (~ 2.8 um diameter)
%%% 3 = M270 beads (~ 2.8 um diameter)
BEADTYPE = 1;

if BEADTYPE == 1

    %%%--- Parameters for MyOne beads ---%%%
    par =  [43.2743   84.4053]; % From Langevin function for to the MyOne bead data
    % First parameter is the saturation magnetization in kA/m
    % Second parameter is the 1/B_0 in 1/T

    scaling_factor = 1.44; % Determined in Lipfert et al. Biophys J 2009
    rad = 1.05/2;          % Radius in um
    vol = 4/3*pi*rad^3;    % Volume in um^3
    vol = scaling_factor * vol;

elseif BEADTYPE == 2

    %%%--- Parameters for M-280, 2.8 mum beads ---%%%
    par =  [13.9650   64.5449]; % From Langevin function for to the M280 bead data
    rad = 2.8/2;
    vol = 4/3*pi*rad^3;
    
elseif BEADTYPE == 3

    %%%--- Parameters for M-270, 2.8 mum beads ---%%%
    par =  [13.9650   64.5449]; % From Langevin function for to the M280 bead data
    par(1) = par(1)*2;          % Scaling factor for M270
    rad = 2.8/2;
    vol = 4/3*pi*rad^3;
    
else
    error(['Unsupported bead type!']);
end




%%% --- Load the PRECOMPUTED field data --- %%%
%%% the script is set up such that you can in principle run over different
%%% parameters, e.g. different gap sizes or different magnet sizes;
%%% Load in data for all conditions with incremented indices "ind"
ind = 1;
dat = load('B_vert_5_5_5_1.txt');
data(ind).B    = dat(:,2)+10^(-20);  % B field in T; add tiny term to avoid singularity
data(ind).zmag = dat(:,1);           % Zmag in mm




%%% Compute the magnetization
for i=1:length(data)
    data(i).magn = vol * (par(1)*(coth(par(2)*data(i).B) - 1./(par(2)*data(i).B)));
end

%%% Calculate the ENERGY 1/2(mB)
for i=1:length(data)
    data(i).energy = 0.5 * data(i).B .* data(i).magn;
end

%%% Calculate the Force: gradient of U (actually the negative gradient, but we essentially take -F)
for i=1:length(data)
    Npoints = length(data(i).magn);

    data(i).force(1) = 0;
    data(i).force(Npoints) = -(data(i).energy(Npoints) - data(i).energy(Npoints-1))/(data(i).zmag(Npoints)-data(i).zmag(Npoints-1));
    for k=2:Npoints-1
        data(i).force(k) = -0.5* ( (data(i).energy(k) - data(i).energy(k-1))/(data(i).zmag(k)-data(i).zmag(k-1)) + (data(i).energy(k+1) - data(i).energy(k))/(data(i).zmag(k+1)-data(i).zmag(k)))
    end

end

%%% Plot force vs. Zmag
if 1
    figure(1); clf; hold on; box on;
    LW = 2;

    col = copper(length(data));
    for ind =1:length(data);
        plot(data(ind).zmag, data(ind).force, '-', 'color', col(ind,:),'linewidth', LW)
    end

    set(gca, 'fontsize', 16, 'linewidth', 1, 'fontweight', 'bold','TickLength',[0.02 0.02]);
    xlabel('Magnet distance (mm)')
    ylabel('Force (pN)')
end

%%% Plot field vs. Zmag
if 1
    figure(2); clf; hold on; box on;
    LW = 2;

    col = copper(length(data));
    for ind =1:length(data);
        plot(data(ind).zmag, data(ind).B, '-', 'color', col(ind,:),'linewidth', LW)
    end

    set(gca, 'fontsize', 16, 'linewidth', 1, 'fontweight', 'bold','TickLength',[0.02 0.02]);
    xlabel('Magnet distance (mm)')
    ylabel('Field (T)')
end


